package generator

import "git.tebibyte.media/sashakoshka/fspl/llvm"
import "git.tebibyte.media/sashakoshka/fspl/entity"

func (this *generator) generateFunction (
	function *entity.Function,
) (
	*llvm.Function,
	error,
) {
	params := make([]*llvm.Parameter, len(function.Signature.Arguments))
	
	ret, err := this.generateType(function.Signature.Return)
	if err != nil { return nil, err }

	for index, argument := range function.Signature.Arguments {
		paramType, err := this.generateType(argument.Type())
		if err != nil { return nil, err }
		params[index] = llvm.NewParameter(argument.Name, paramType)
	}
	
	var name string; if function.LinkName == "" {
		name = function.Signature.Name
	} else {
		name = function.LinkName
	}
	irFunc := this.module.NewFunction(name, ret, params...)

	if function.Body != nil {
		this.blockManager = this.pushBlockManager(irFunc)
		defer this.popBlockManager()
		
		this.blockManager.addFunctionArgumentDeclarations(function)
		body, err := this.generateExpressionVal(function.Body)
		if err != nil { return nil, err }

		if function.Signature.Return == nil {
			this.blockManager.NewRet(nil)
		} else {
			this.blockManager.NewRet(body)
		}
	}
	
	this.functions[function.Signature.Name] = irFunc
	return irFunc, nil
}

func (this *generator) generateMethod (
	method *entity.Method,
) (
	*llvm.Function,
	error,
) {
	params := make([]*llvm.Parameter, len(method.Signature.Arguments) + 1)
	
	ret, err := this.generateType(method.Signature.Return)
	if err != nil { return nil, err }

	for index, argument := range method.Signature.Arguments {
		paramType, err := this.generateType(argument.Type())
		if err != nil { return nil, err }
		params[index + 1] = llvm.NewParameter(argument.Name, paramType)
	}
	params[0] = llvm.NewParameter("this", new(llvm.TypePointer))
	
	var name string; if method.LinkName == "" {
		name = method.TypeName + "." + method.Signature.Name
	} else {
		name = method.LinkName
	}
	irFunc := this.module.NewFunction(name, ret, params...)

	if method.Body != nil {
		this.blockManager = this.pushBlockManager(irFunc)
		defer this.popBlockManager()
		
		this.blockManager.addFunctionArgumentDeclarations(method)
		body, err := this.generateExpressionVal(method.Body)
		if err != nil { return nil, err }

		if method.Signature.Return == nil {
			this.blockManager.NewRet(nil)
		} else {
			this.blockManager.NewRet(body)
		}
	}
	
	this.functions[method.TypeName + "." + method.Signature.Name] = irFunc
	return irFunc, nil
}
